UNIT WWIV;

{*****************************************************************}
{  Interface between  WWIV v4.10 BBS system and on-line programs  }
{                                                                 }
{  Source code dedicated to public domain by the author, Jeff M.  }
{  Garzik on January 26, 1990.  Any person may use this software  }
{  for  any  length  of  time without  fee or  donation  to  the  }
{  afore-mentioned author.  The author  hereby disclaims any and  }
{  all liability, or warranty, expressed  or implied, that might  }
{  possibly be associated  with this  product,  or with its use.  }
{  The  author is  in  NO WAY  responsible  for any  damage this  }
{  product may cause.                                             }
{*****************************************************************}
{  FOSSIL DRIVER INTERFACE FOR WWIV 1.0 by 1@7310                 }
{                                                                 }
{  Modified 03/91 by Hollie Satterfield to optionally use a       }
{  fossil driver and to use real ANSI instead of WWIV ANSI.       }
{  This will allow WWIV sysops to run games the way they always   }
{  have, but make games more portable to other BBS systems.       }
{                                                                 }
{  The ansic() function, which used to use WWIV ANSI, now uses    }
{  real ANSI to generate WWIV default colors.  Look at ANSI.PAS   }
{  and ansic() if you want to see how to do your own colors.      }
{                                                                 }
{  If the environment variable FOSSIL=TRUE then the program will  }
{  try to access a fossil driver.  If not, it will run using      }
{  standard I/O the way it always did.  Note that, if you want to }
{  use a fossil driver under WWIV, DOS interrupts must be OFF.    }
{                                                                 }
{  Fossil driver function calls derived from the fossil revision  }
{  5 documentation and should work with any proper fossil driver. }
{  Tested using BNU 1.70 fossil driver, a freeware product.       }
{*****************************************************************}

{$DEFINE Ver410}
{$IFDEF Debug}
{$R+}
{$S+}
{$D+}
{$L+}
{$V+}
{$ELSE}
{$R-}
{$S-}
{$D-}
{$L-}
{$V-}
{$ENDIF}
{$I+} { Enable I/O checking }
{$F-} { Disable forced far calls }
{$B-} { Enable short-circuit boolean evaluation }
{$N-} { Disable 8087/80287 compress-dependant compilation }

Interface

Uses
  DOS, Fossil;    (* requires FOSSIL.PAS file compiled into FOSSIL.TPU *)

CONST
  Version_Number = 'WWIV v4.1x';

TYPE
  SystemRec = RECORD
    BaudRate  : STRING[5];  { baud rate user is at      }
    Port      : BYTE;       { comport currently in use  }
    Name      : STRING[51]; { system name               }
    Sysop     : STRING[51]; { sysop's name              }
    TimeOn    : LONGINT;    { time user logged on       }
    TimeUsed  : LONGINT;    { time used so far          }
    Uk        : WORD;       { # of k uploaded by user   }
    Uploads   : WORD;       { # of uploads by user      }
    Dk        : WORD;       { # of k downloaded by user }
    Downloads : WORD;       { # of downloads by user    }
    Databits  : BYTE;       { databits modem is set to  }
    Stopbits  : BYTE;       { stop bits modem is set to }
    Parity    : CHAR;       { parity modem is set to    }
  END;

  UserRec = RECORD
    Name     : STRING[30];  { user's alias              }
    RealName : STRING[14];  { user's real name          }
    LastOn   : STRING[10];  { date user was last on     }
    LineLen  : BYTE;        { # of columns              }
    PageLen  : BYTE;        { # of screen rows          }
    SL       : BYTE;        { security level            }
    Age      : BYTE;        { age                       }
    Sex      : CHAR;        { sex ('F' or 'M')          }
    Callsign : STRING[8];   { amateur radio callsign    }
    Gold     : REAL;        { game money                }
  END;

VAR
  SysConfg    : SystemRec;
  SysopF      : TEXT;
  SysopFFN    : STRING[80];
  GFilesPath,
  DataPath    : STRING[80];
  Usernum     : integer;
  Incom,
  OkANSI,
  cs,
  so,
  timedout    : BOOLEAN;
  TimeOn,
  TimeLeft    : REAL;
  Thisuser    : userrec;
  Regs        : Registers;
  logflag     : BOOLEAN;
  UseFossil   : boolean;

procedure Ansic(c : integer);
procedure Background(bgnd : integer); (* new *)
procedure Centre(VAR i : STRING);
procedure Cls;
function  Cstr(i : WORD) : STRING;
function  Cstrr(rl : REAL; base : integer) : STRING;
function  CTim(rl : REAL) : STRING;
function  Dat : STRING;
function  Date : STRING;
function  DayCount(mo, yr : integer) : integer;
function  DayNum(dt : STRING) : integer;
function  Days(mo, yr : integer) : integer;
function  Empty : BOOLEAN;
procedure endANSI; (* new *)
function  Exist(fln : string) : BOOLEAN;
procedure Foreground(fgnd : integer); (* new *)
procedure GetKey(var c : char);
function  Hangup : boolean; (* new *)
function  Inkey : CHAR;
procedure Input(VAR i : STRING; ml : integer);
procedure Input1(VAR i : STRING; ml : integer; tf : BOOLEAN);
procedure InputL(VAR i : STRING; ml : integer);
procedure InterfaceExitProcedure;
function  LeapYear(yr : integer) : BOOLEAN;
procedure Mpl(c : integer);
function  Nam : STRING;
procedure Nl;
function  Nsl : REAL;
function  OneK(ch : STRING) : CHAR;
procedure PauseSCR;
procedure Pfl(fn : STRING; VAR Abort : BOOLEAN; cr : BOOLEAN);
procedure Prompt(st : STRING);
procedure Print(st : STRING);
procedure PrintA(i : STRING; VAR Abort, Next : BOOLEAN);
procedure PrintA1(i : STRING; VAR Abort, Next : BOOLEAN);
procedure PrintACR(i : STRING; VAR Abort, Next : BOOLEAN);
procedure PrintFile(fn : STRING);
procedure Prt(Color : BYTE; i : STRING);
procedure ReadParmFile;
function  Timer : REAL;
function  TLef : STRING;
procedure TLeft;
procedure Sl1(i : STRING);
function  Sysop1 : BOOLEAN;
procedure SysopLog(i : STRING);
function  Tch(i : STRING) : STRING;
function  Time : STRING;
function  Value(I : STRING) : WORD;
procedure WKey(VAR Abort, Next : BOOLEAN);
function  Yn : BOOLEAN;


Implementation

VAR
  ExitSave : Pointer;

(**************************************************************************)
(* All loops which do screen I/O should abort when this function is true. *)
(* Otherwise your program may go into an infinite loop or lock up when    *)
(* someone hangs up.  This function is true when either there is no       *)
(* carrier or the user has run out of time.                               *)
(**************************************************************************)
function Hangup : boolean;
var tf:boolean;
begin
  if incom
    then tf:=(timedout or NoCarrier(portused))
    else tf:=timedout;
  if tf
    then halt
    else hangup:=false;
end;

procedure InterfaceExitProcedure;
BEGIN
  {$I-} Close(SysopF); {$I+}
  ExitProc := ExitSave;
END;

function Exist(fln : string) : BOOLEAN;
VAR
  S : SearchRec;
BEGIN
  FindFirst(Fln, AnyFile, S);
  Exist := (DosError = 0);
END;

function Timer;
VAR
  Reg : Registers;
  h, m, s, t  : REAL;
BEGIN
  Reg.AX := 44 * 256;
  MSDos(Reg);
  h := Reg.CH;
  m := Reg.CL;
  s := Reg.DH;
  t := Reg.DL;
  Timer := h*3600 + m*60 + s + t/100;
END;

function Nsl;
BEGIN
  IF Timer < TimeOn THEN
    TimeOn := TimeOn - 24.0 * 3600.0;
  Nsl := TimeLeft - (Timer - TimeOn);
END;

function Sysop1; (* checks scroll lock for sysop availability *)
BEGIN
  Sysop1 := ((Mem[0:1047] AND 16) <> 0);
END;

procedure Sl1;
BEGIN
  WriteLN(SysopF, I);
END;

procedure SysopLog;
BEGIN
 IF (NOT so) or logflag THEN
   Sl1('       ' + I);
END;

function Tch;
BEGIN
  IF Length(i) > 2 THEN
    i := copy(i, Length(i) - 1, 2)
  ELSE
    IF Length(i) = 1 THEN
      i := '0' + i;
  Tch := i;
END;

function Time;
VAR
  Reg : Registers;
  h, m, s : STRING[4];
BEGIN
  Reg.ax := $2c00;
  intr($21, Reg);
  Str(Reg.ch, h);
  Str(Reg.cl, m);
  Str(Reg.dh, s);
  Time := Tch(h) + ':' + Tch(m) + ':' + Tch(s);
END;

function Date : STRING;
VAR
  Reg : Registers;
  m, d, y : STRING[4];
BEGIN
  Reg.ax := $2a00;
  msdos(Reg);
  Str(Reg.dh, m);
  Str(Reg.dl, d);
  Str(Reg.cx, y);
  Date := Tch(m) + '/' + Tch(d) + '/' + Tch(y);
END;

function Value(I : STRING) : WORD;
VAR
  n,
  n1 : integer;
BEGIN
  val(i, n, n1);
  IF n1 <> 0 THEN
    BEGIN
      i := copy(i, 1, n1 - 1);
      val(i, n, n1)
    END;
  Value := n;
  IF i = '' THEN
    Value := 0;
END;

function Cstr(i : WORD) : STRING;
VAR
  c : STRING;
BEGIN
  Str(i, c);
  Cstr := c;
END;

function Nam : STRING;
VAR
  s  : STRING;
  i  : integer;
  tf : BOOLEAN;
BEGIN
  s := Thisuser.name;
  tf := TRUE;
  FOR i := 1 to Length(s) DO
    IF s[i] < 'A' THEN
      tf := TRUE
    ELSE
      BEGIN
        IF (s[i] <= 'Z') AND NOT tf THEN
          s[i] := chr(ord(s[i]) + 32);
        tf := FALSE;
      END;
  Nam := s + ' #' + Cstr(Usernum);
END;

function LeapYear(yr : integer) : BOOLEAN;
BEGIN
  LeapYear := (yr mod 4 = 0) AND
             ((yr mod 100 <> 0) or
              (yr mod 400 = 0));
END;

function Days(mo, yr : integer) : integer;
VAR
  d : integer;
BEGIN
  d := Value(copy('312831303130313130313031',1 + (mo - 1) * 2, 2));
  IF (mo = 2) AND LeapYear(yr) THEN
    inc(d);
  Days := d;
END;

function DayCount(mo, yr : integer) : integer;
VAR
  m,
  t : integer;
BEGIN
  t := 0;
  FOR m := 1 to (mo - 1) DO
    t := t + Days(m, yr);
  DayCount := t;
END;

function DayNum(dt : STRING) : integer;
VAR
  d, m,
  y, t,
  c : integer;
BEGIN
  t := 0;
  m := Value(copy(dt, 1, 2));
  d := Value(copy(dt, 4, 2));
  y := Value(copy(dt, 7, 2)) + 1900;
  FOR c := 1985 to (y - 1) DO
    IF LeapYear(c) THEN
      inc(t, 366)
    ELSE
      inc(t, 365);
  t := t + DayCount(m, y) + (d - 1);
  DayNum := t;
  IF y < 1985 THEN
    DayNum := 0;
END;

function Dat : STRING;
VAR
  ap,
  x,
  y  : STRING;
  i  : integer;
BEGIN
  case DayNum(Date) mod 7 of
    0 : x := 'Tue';
    1 : x := 'Wed';
    2 : x := 'Thu';
    3 : x := 'Fri';
    4 : x := 'Sat';
    5 : x := 'Sun';
    6 : x := 'Mon';
  END;
  case Value(copy(Date,1,2)) of
    1 : y := 'Jan';
    2 : y := 'Feb';
    3 : y := 'Mar';
    4 : y := 'Apr';
    5 : y := 'May';
    6 : y := 'Jun';
    7 : y := 'Jul';
    8 : y := 'Aug';
    9 : y := 'Sep';
   10 : y := 'Oct';
   11 : y := 'Nov';
   12 : y := 'Dec';
  END;
  x := x + ' ' + y + ' ' + copy(Date,4,2) + ', ' +
    Cstr(1900 + Value(copy(Date, 7, 2)));
  y := Time;
  i := Value(copy(y, 1, 2));
  IF i > 11
    THEN ap := 'pm'
    ELSE ap := 'am';
  IF i > 12 THEN
    dec(i, 12);
  IF i = 0 THEN
    i := 12;
  Dat := Cstr(i) + copy(y, 3, 3) + ' ' + ap + '  '+ x;
END;

{$I ANSI.PAS}

procedure PauseSCR;
VAR
  i  : integer;
  cc : CHAR;
BEGIN
  Ansic(3);
  Prompt('(-*-)');
  Ansic(0);
  GetKey(cc);
  FOR i := 1 to 5 DO
    Prompt(#8#32#8);
END;

procedure Prompt;
VAR
  c : BYTE;
BEGIN
  c := 1;
  while (c<=length(st)) and not hangup do begin
    IF (st[c]=#10) THEN
      Ansic(0);
    if incom
      then XmitPort(byte(st[c]), portused);
    write(st[c]);
    c:=c+1;
  end;
END;

procedure Print;
VAR
  C : BYTE;
BEGIN
  st := st + #13 + #10;
  Prompt(st);
END;

procedure Nl;
BEGIN
  Prompt(chr(13)+chr(10))
END;

procedure Prt;
BEGIN
  Ansic(Color);
  Prompt(i);
  Ansic(0);
END;

procedure Mpl(c:integer);
VAR
  n : integer;
  i : STRING;
BEGIN
  IF OkANSI THEN
    BEGIN
      Ansic(6);
      i := '';
      FOR n := 1 to c DO
        i := i + ' ';
      Prompt(i);
      Prompt(#27 + '[' + Cstr(c) + 'D');
    END;
END;

procedure TLeft;
BEGIN
 IF Timer < TimeOn THEN
   TimeOn := TimeOn - 24.0 * 60 * 60;
 IF (Nsl<0) THEN
   BEGIN
     Nl;
     Print('Time expired.');
     timedout := TRUE;
   END;
END;

(************************************************************)
(* The empty function must now check both the modem and the *)
(* keyboard for input.  This function checks the modem.     *)
(************************************************************)
function portempty:boolean;
begin
  if incom
    then portempty := not PortDataRdy(portused)
    else portempty := true;
end;

(************************************************************)
(* The empty function must now check both the modem and the *)
(* keyboard for input.  This function checks the keyboard.  *)
(************************************************************)
function keybdempty:boolean;
var rp:registers;
begin
  rp.ax:=$0b00;
  msdos(rp);
  if (rp.al = 0)
    then keybdempty:=true
    else keybdempty:=false;
end;

function Empty;
BEGIN
  empty := portempty and keybdempty;
END;

function Inkey : CHAR;
VAR
  ch : CHAR;  rp:registers;
BEGIN
  IF (Empty) THEN
    Inkey := #0
  ELSE BEGIN
    if portempty then begin (* If the port is empty, read the keyboard. *)
      rp.ax := $0800;
      msdos(rp);
      inkey := chr(rp.al);
    end
    else begin           (* else read a character from the serial port. *)
      inkey := char(RcvPort(portused));
    end;
  END;
END;

procedure GetKey;
BEGIN
  repeat
    c := Inkey;
  until hangup or (c<>chr(0));
END;

procedure Cls;
BEGIN
  Prompt(#12);
END;

function Yn;
VAR
  c : CHAR;
BEGIN
  IF NOT Hangup THEN
    BEGIN
      Ansic(3);
      REPEAT
        GetKey(c);
        c := upcase(c);
      UNTIL (c = 'Y') or (c = 'N') or (c = #13) or Hangup;
      IF c = 'Y' THEN
        BEGIN
          Print('Yes');
          Yn := TRUE;
        END
      ELSE
        BEGIN
          Print('No');
          Yn := FALSE;
        END;
      IF Hangup THEN
        Yn := FALSE;
    END;
END;

procedure Input1;
VAR
  cp : integer;
  c  : CHAR;
  r  : REAL;
BEGIN
  IF NOT Hangup THEN
    BEGIN
      r := Timer;
      cp := 1;
      REPEAT
        GetKey(c);
        IF c = #1 THEN
          r := Timer;
        IF NOT tf THEN
          c := upcase(c);
        IF (c >= ' ') AND (c < #127) THEN
          IF cp <= ml THEN
            BEGIN
              i[cp] := c;
              inc(cp);
              write(c);
              if incom (* HHS *)
                then XmitPort(byte(c), portused);
            END
          ELSE
        ELSE
          case ord(c) of
            8   : IF cp > 1 THEN
                    BEGIN
                      Prompt(#8#32#8);
                      dec(cp);
                    END;
            21,
            24  : WHILE cp <> 1 DO
                    BEGIN
                      dec(cp);
                      Prompt(#8#32#8);
                    END;
          END;
        IF (Timer - r) > 300.0 THEN
          timedout := TRUE;
      UNTIL (c = #13) or (c = #14) or Hangup;
      i[0] := chr(cp - 1);
      Nl;
    END;
END;

procedure Input;
BEGIN
  Input1(i, ml, FALSE);
END;

procedure InputL;
BEGIN
  Input1(i, ml, TRUE);
END;

function OneK;
VAR
  C : CHAR;
BEGIN
  IF Ch = '' THEN
    BEGIN
      OneK := #0;
      Exit;
    END;
  REPEAT
    GetKey(c);
    c := upcase(c);
  UNTIL (pos(c,ch) > 0) or Hangup;
  IF Hangup THEN
    c := ch[1];
  Print('' + c);
  OneK := c;
END;

procedure Centre;
VAR
  n,
  n1 : integer;
BEGIN
  IF i[1] = #2 THEN
    i := copy(i, 2, Length(i)-1);
  n := Length(i);
  n1 := 1;
  WHILE (n1 <= Length(i)) DO
    BEGIN
      IF i[n1] = #3 THEN
        BEGIN
          dec(n, 2);
          inc(n1);
        END;
      inc(n1);
    END;
  IF n<Thisuser.linelen THEN
    i:=copy('                                               ',1,
           (Thisuser.linelen - n) div 2) + i;
END;

procedure WKey;
VAR
  cc : CHAR;
BEGIN
  WHILE NOT (Empty or Hangup or Abort) DO
    BEGIN
      GetKey(cc);
      IF (cc = ' ') or (cc = #3) or (cc = #24) or (cc = #11) THEN
        Abort := TRUE;
      IF (cc = #14) THEN
        BEGIN
          Abort := TRUE;
          Next := TRUE;
        END;
      IF (cc = #19) or (cc = 'P') or (cc = 'p') THEN
        GetKey(cc);
    END;
END;

function CTim;
VAR
  h,
  m,
  s  : STRING;
BEGIN
  s := Tch(Cstr(trunc(rl-int(rl/60.0)*60.0)));
  m := Tch(Cstr(trunc(int(rl/60.0)-int(rl/3600.0)*60.0)));
  h := Cstr(trunc(rl/3600.0));
  IF Length(h) = 1 THEN
    h := '0' + h;
  CTim := h + ':' + m + ':' + s;
END;

function TLef;
BEGIN
  TLef := CTim(Nsl);
END;

function Cstrr;
VAR
  c1,
  c2,
  c3  : integer;
  i   : STRING;
  r1,
  r2  : REAL;
BEGIN
  IF rl <= 0.0 THEN
    Cstrr := '0'
  ELSE
    BEGIN
      r1 := ln(rl) / ln(1.0 * base);
      r2 := exp(ln(1.0*base) * (trunc(r1)));
      i := '';
      WHILE (r2 > 0.999) DO
        BEGIN
          c1 := trunc(rl / r2);
          i := i + copy('0123456789ABCDEF',c1 + 1,1);
          rl := rl - c1 * r2;
          r2 := r2 / (1.0 * base);
        END;
      Cstrr := i;
    END;
END;

procedure PrintA1;
VAR
  c : integer;
BEGIN
  IF NOT Hangup THEN
    BEGIN
      Abort := FALSE;
      Next := FALSE;
      c := 1;
      IF NOT Empty THEN
        WKey(Abort, Next);
      WHILE (NOT Abort) AND (c-1 < Length(i)) AND (NOT Hangup) DO
        BEGIN
          IF i[c]=#3 THEN
            IF i[c + 1] in [#0..#8] THEN
              IF OkANSI THEN
                Ansic(ord(i[c + 1]));
          IF NOT Empty THEN
            WKey(Abort, Next);
          IF i[c] = #3 THEN
            inc(c)
          ELSE BEGIN
            write(i[c]);
            if incom
              then XmitPort(byte(c), portused);
          END;
          inc(c);
        END;
    END
  ELSE
    Abort := TRUE;
END;

function WhereX : BYTE;
VAR
  Regs : Registers;
BEGIN
  Regs.AH := $03;
  Intr($10, Regs);
  WhereX := Regs.DL;
END;

procedure PrintA;
VAR
  s     : STRING;
  p,
  op,
  Regs,
  rop,
  nca   : integer;
  crend : BOOLEAN;
BEGIN
  Abort := FALSE;
  crend := (i[Length(i)] = #1) AND (i[Length(i)-1] <> #3);
  IF crend THEN
    i := copy(i, 1, Length(i)-1);
  WKey(Abort, Next);
  IF i = '' THEN
    Nl;
  WHILE (i <> '') AND (NOT Abort) AND (NOT Hangup) DO
    BEGIN
      Regs := 0;
      nca := Thisuser.linelen - wherex - 1;
      p := 0;
      WHILE (Regs < nca) AND (p < Length(i)) DO
        BEGIN
          IF i[p+1]=#8 THEN
            dec(Regs)
          ELSE
            IF i[p+1] = #3 THEN
              inc(p)
            ELSE
             IF (i[p+1]<>#10) THEN
               inc(Regs);
          inc(p);
        END;
      op := p;
      rop := Regs;
      IF (Regs >= nca) AND (p < Length(i)) THEN
        BEGIN
          WHILE ((NOT (i[p] in [' ',#8,#10])) or (i[p-1] = #3)) AND (p > 1) DO
            BEGIN
              dec(regs);
              dec(p);
            END;
          IF p = 1 THEN
          IF NOT (i[1] in [' ',#8,#10]) THEN
            BEGIN
              dec(Regs);
              dec(p);
            END;
        END;
      IF abs(rop-Regs) >= (Thisuser.linelen div 2) THEN
        p := op;
      s := copy(i, 1, p);
      delete(i, 1, p);
      IF (s[Length(s)] = ' ') THEN
        s[0] := pred(s[0]);
      PrintA1(s, Abort, Next);
      IF ((i = '') AND crend) or (i <> '') or Abort THEN
        Nl
      ELSE
        PrintA1(' ', Abort, Next);
    END;
END;

procedure PrintACR;
BEGIN
  IF NOT Abort THEN
    IF i[Length(i)] = #1 THEN
      PrintA(i, Abort, Next)
    ELSE
      PrintA(i + #1, Abort, Next);
END;

(* I have corrected this procedure so that it will print ANSI files
   correctly.  The other procedure was designed to print files
   according to how many columns the user had, so ANSI files didn't
   work.   Parameter cr, which flags whether or not to insert
   carriage returns, is a stupid idea, and is ignored. *)
procedure Pfl(fn : STRING; VAR Abort : BOOLEAN; cr : BOOLEAN);
VAR
  fil  : TEXT;
  Next : BOOLEAN;
  c    : CHAR;
BEGIN
  IF NOT Hangup THEN BEGIN
    Assign(fil, fn);
    {$I-} Reset(fil); {$I+}
    IF ioresult <> 0 THEN
      Print('File NOT found.')
    ELSE BEGIN
      Abort := FALSE;
      WHILE NOT eof(fil) AND (NOT Abort) AND (NOT Hangup) DO
        BEGIN
          Read(fil, c);
          write(c);
          if incom then XmitPort(byte(c), portused);
          if not empty then wkey(abort, next);
        END;
      Close(fil);
    END;
(*  Nl;  These extra CRLFs are also annoying, the file should be
    Nl;  formatted the way you want it to show up. *)
  END;
END;

procedure PrintFile(fn : STRING);
VAR
  Abort : BOOLEAN;
BEGIN
  Abort := FALSE;
  Pfl(fn, Abort, TRUE);
END;

function capitalize(ins:string):string;
var i:integer;
    s:string;
begin
  s := ins;
  for i:=1 to length(s) do
    if (s[i] >= 'a') and (s[i] <= 'z')
      then s[i] := chr(ord(s[i])-32);
  capitalize := s;
end;

procedure ReadParmFile;
VAR
  f : TEXT;
  i : STRING;
  n : integer;
BEGIN
  IF paramcount < 1  THEN
    BEGIN
      WriteLN('You must specify parameter file name (ie. CHAIN.TXT).');
      halt(1);
    END;
  Assign(f, paramstr(1));
  {$I-} Reset(f); {$I+}
  IF (ioresult = 0) THEN
    BEGIN
      ReadLN(f, Usernum);
      ReadLN(f, Thisuser.name);
      ReadLN(f, Thisuser.realname);
      ReadLN(f, Thisuser.callsign);
      ReadLN(f, Thisuser.age);
      ReadLN(f, Thisuser.sex);
      ReadLN(f, Thisuser.gold);
      ReadLN(f, Thisuser.laston);
      ReadLN(f, Thisuser.linelen);
      ReadLN(f, Thisuser.pagelen);
      ReadLN(f, Thisuser.sl);
      ReadLN(f, n);
      cs := (n = 1);
      ReadLN(f, n);
      so := (n = 1);
      ReadLN(f, n);
      OkANSI := (n = 1);
      ReadLN(f, n);
      Incom := (n = 1);
      logflag := Incom; (* HHS *)
      ReadLN(f, TimeLeft);
      ReadLN(f, GFilesPath);
      ReadLN(f, DataPath);
      ReadLN(f, i);
      SysopFFN := GFilesPath + i;
      ReadLN(f, sysconfg.baudrate);
      ReadLN(f, sysconfg.Port);
      case sysconfg.Port of
        2 : portused:=1;
        3 : portused:=2;
        4 : portused:=3;
        else portused:=0;
      end;
      ReadLN(f, sysconfg.name);
      ReadLN(f, sysconfg.Sysop);
      ReadLN(f, sysconfg.TimeOn);
      ReadLN(f, sysconfg.timeused);
      ReadLN(f, sysconfg.uk);
      ReadLN(f, sysconfg.uploads);
      ReadLN(f, sysconfg.dk);
      ReadLN(f, sysconfg.downloads);
      ReadLN(f, i);
      Close(f);
      sysconfg.databits := Value(copy(i, 1, 1));
      sysconfg.stopbits := Value(copy(i, 3, 1));
      sysconfg.parity := i[2];
      Assign(SysopF, SysopFFN);
      {$I-} append(SysopF); {$I+}
      IF (ioresult <> 0) THEN
        rewrite(SysopF);
    END
  ELSE
    BEGIN
      WriteLN('Parameter file NOT found/read (ie. CHAIN.TXT).');
      halt(1);
    END;
  UseFossil := (capitalize(GetEnv('FOSSIL')) = 'TRUE');
  if UseFossil and incom then begin
    InitFossil(portused);
    if not FossilInstalled
      then begin
        writeln('FOSSIL not Found - critical Error');
        halt;
      end;
  end
  else incom:=false;
  timedout := FALSE;
  TimeOn := Timer;
END;

BEGIN
  ExitSave := ExitProc;
  ExitProc := @InterfaceExitProcedure;
  timedout := FALSE;
END.
